<?php

/***************************************************************
 *  Copyright notice
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
namespace SGalinski\SgNews\Preview;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * small helper class, aslong as we use PageLayout hook + Previewrenderer....
 */
class PreviewService {
	public const RETURNTYPE_ARR = 'array';

	/**
	 * returns a Standalone view for SgNews Overview Plugin
	 *
	 * @param array $row
	 * @return StandaloneView
	 */
	public function getOverviewView(array $row): StandaloneView {
		$view = $this->createViewWithTemplate('Overview');
		$view->assign('uid', $row['uid']);

		// Get available plugin settings and their values from flexform
		$pluginConfiguration = GeneralUtility::xml2array(
			$row['pi_flexform'],
			'T3DataStructure'
		)['data']['main']['lDEF'];

		$templateData = [
			'groupBy' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.groupBy'),
			'enableFilter' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.enableFilter'),
			'newsLimit' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.newsLimit'),
			'onlyNewsWithinThisPageSection' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.onlyNewsWithinThisPageSection'),
			'starttime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.starttime'),
			'endtime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.endtime'),
			'sortBy' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.sortBy'),
			'sortDirection' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.sortDirection'),
		];

		$view->assign('data', $templateData);
		return $view;
	}

	/**
	 * Returns LatestView for Previews of sg_news
	 *
	 * @param array $row
	 * @return StandaloneView
	 */
	public function getLatestView(array $row): StandaloneView {
		$view = $this->createViewWithTemplate('Latest');
		$view->assign('uid', $row['uid']);

		// Get available plugin settings and their values from flexform
		$pluginConfiguration = GeneralUtility::xml2array(
			$row['pi_flexform'],
			'T3DataStructure'
		)['data']['main']['lDEF'];

		$categories = $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.categories');
		$tags = $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.tags');
		$templateData = [
			'limit' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.limit'),
			'categories' => is_string($categories) ? $this->addFieldContentsToRecordIdList(
				'pages',
				$categories
			) : '',
			'tags' => is_string($tags) ? $this->addFieldContentsToRecordIdList(
				'sys_category',
				$tags
			) : '',
			'starttime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.starttime'),
			'endtime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.endtime'),
			'sortBy' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.sortBy'),
		];

		$view->assign('data', $templateData);
		return $view;
	}

	/**
	 * Returns ListByCategory View for Previews of sg_news
	 *
	 * @param array $row
	 * @return StandaloneView
	 */
	public function getListByCategoryView(array $row): StandaloneView {
		$view = $this->createViewWithTemplate('ListByCategory');
		$view->assign('uid', $row['uid']);


		// Get available plugin settings and their values from flexform
		$pluginConfiguration = GeneralUtility::xml2array(
			$row['pi_flexform'],
			'T3DataStructure'
		)['data']['main']['lDEF'];

		$categories = $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.categories');
		$tags = $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.tags');
		$templateData = [
			'newsLimitPerPage' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.newsLimitPerPage'),
			'categories' => is_string($categories) ? $this->addFieldContentsToRecordIdList(
				'pages',
				$categories
			) : '',
			'tags' => is_string($tags) ? $this->addFieldContentsToRecordIdList(
				'sys_category',
				$tags
			) : '',

			'starttime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.starttime'),
			'endtime' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.endtime'),
			'sortBy' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.sortBy'),
			'sortDirection' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.sortDirection')
		];

		$view->assign('data', $templateData);
		return $view;
	}

	/**
	 * Returns News by author View for Previews of sg_news
	 *
	 * @param array $row
	 * @return StandaloneView
	 */
	public function getNewsByAuthorView(array $row): StandaloneView {
		$view = $this->createViewWithTemplate('NewsByAuthor');
		$view->assign('uid', $row['uid']);

		// Get available plugin settings and their values from flexform
		$pluginConfiguration = GeneralUtility::xml2array(
			$row['pi_flexform'],
			'T3DataStructure'
		)['data']['main']['lDEF'];

		$newsAuthors = $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.newsAuthors');
		$templateData = [
			'showDetails' => $this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.showDetails'),
			'newsAuthors' => is_string($newsAuthors) ? $this->addFieldContentsToRecordIdList(
				'tx_sgnews_domain_model_author',
				$newsAuthors,
				'name'
			) : ''
		];

		// Not using addFieldContentsToRecordIdList to avoid repetitive imploding and exploding.
		$excludedNewsIds = GeneralUtility::intExplode(
			',',
			$this->passVDefOnKeyToTemplate($pluginConfiguration, 'settings.excludedNews'),
			TRUE
		);
		$excludedNewsListWithTitles = [];
		foreach ($excludedNewsIds as $excludedNewsId) {
			$excludedNewsListWithTitles[] = BackendUtility::getRecord(
				'pages',
				$excludedNewsId,
				'title'
			)['title'] . ' [' . $excludedNewsId . ']';
		}
		$templateData['excludedNews'] = $excludedNewsListWithTitles;

		$view->assign('data', $templateData);
		return $view;
	}

	/**
	 * @param array $conf
	 * @param string $key
	 * @param string $returnType
	 * @return array|mixed|string
	 */
	private function passVDefOnKeyToTemplate(array $conf, string $key, string $returnType = '') {
		if (isset($conf[$key])) {
			return $conf[$key]['vDEF'];
		}

		// check if we got a possible returntype:
		if ($returnType === self::RETURNTYPE_ARR) {
			return [];
		}

		return '';
	}

	/**
	 * Creates a new StandaloneView object with template and partial root paths,
	 * attaches the template with the given name to it and returns it.
	 *
	 * @param string $templateName
	 * @return StandaloneView
	 */
	protected function createViewWithTemplate(string $templateName): StandaloneView {
		$view = GeneralUtility::makeInstance(StandaloneView::class);
		$view->setTemplateRootPaths(['EXT:sg_news/Resources/Private/Templates/Backend']);
		$view->setPartialRootPaths(['EXT:sg_news/Resources/Private/Partials/Backend']);
		if (!str_ends_with($templateName, '.html')) {
			$templateName .= '.html';
		}
		$view->setTemplate($templateName);
		return $view;
	}

	/**
	 * Takes a comma-separated list of record IDs, the corresponding table and optionally the field to look up.
	 * Returns another comma-space-separated list of the same records with the content of the field added.
	 * The returned list should look nice enough to be rendered in the backend preview directly.
	 *
	 * @param string $table
	 * @param string $recordIdList
	 * @param string $field
	 * @return string
	 */
	protected function addFieldContentsToRecordIdList(
		string $table,
		string $recordIdList,
		string $field = 'title'
	): string {
		$recordIdsArray = GeneralUtility::intExplode(',', $recordIdList, TRUE);
		$recordsWithTitlesArray = [];

		foreach ($recordIdsArray as $recordId) {
			$recordsWithTitlesArray[] = BackendUtility::getRecord(
				$table,
				$recordId,
				$field
			)[$field] . ' [' . $recordId . ']';
		}
		return implode(', ', $recordsWithTitlesArray);
	}
}
